<?php


namespace MetatickPluginLib;

use MetatickPluginLib\License\License;
use Puc_v4_Factory;

class Plugin
{
    /*
    Each plugin must implement the following:

    static $config = null;
    static $instance = null;
    static $license = null;
    static $template_loader = null;
    static $template_engine = null;
    */

    //these remain here too for legacy reasons
    static $config = null;
    static $instance = null;
    static $license = null;
    static $template_loader = null;
    static $template_engine = null;

    public static function instance(){
        if(!static::$instance){
            static::$instance = new static();
        }
        return static::$instance;
    }

    public static function run(){
        static::update_checker();
        static::instance();
    }

    public static function update_checker()
    {
        $config = static::config();
        $url = strtr($config->UPDATE_URL, ['$slug' => static::SLUG]);
        Puc_v4_Factory::buildUpdateChecker(
            $url,
            static::FILE,
            static::SLUG
        );
    }

    public static function path($path){
        return join(
            '/',
            [
                rtrim(static::PATH, '/'),
                ltrim($path, '/')
            ]
        );
    }

    public static function assets_url($path){
        return static::plugins_url(['assets', $path]);
    }

    public static function plugins_url($path){
        if(is_array($path)){
            $path = join('/', $path);
        }
        return plugins_url($path, static::FILE);
    }

    public static function config(){
        if(!static::$config){
            static::$config = json_decode(file_get_contents(static::PATH . '/.config'));
        }
        return static::$config;
    }

    public static function license(){
        if(!static::$license){
            $config = static::config();
            $host = $config->LICENSE_HOST;
            $secret = $config->LICENSE_SECRET;
            $type = $config->LICENSE_TYPE;
            static::$license = new License($host, static::NAME, $secret, static::SLUG, $type === 'network');
        }
        return static::$license;
    }
}