import os
from zipfile import ZipFile, ZipInfo, ZIP_DEFLATED

FILTERED = (
    '.git',
    '.gitignore',
    '.gitmodules',
    '.idea',
    '.private',
    '.development',
    'composer.json',
    'composer.lock',
    '.gitlab-ci.yml',
    '.license',
    'config',
    'dev'
)


def is_filtered(path):
    parts = path.replace('\\', '/').split('/')
    for filter in FILTERED:
        if filter in parts:
            return True

    return False


def create_package(archive_path):
    plugin_name = os.getcwd().replace('\\', '/').split('/')[-1]
    folders_to_create = []
    files_to_add = []

    for root, sub_folders, files in os.walk('.'):

        for folder in sub_folders:
            path = str(os.path.join(root, folder))
            if not is_filtered(path):
                folders_to_create.append(path)

        for file in files:
            path = str(os.path.join(root, file))
            if not is_filtered(path):
                files_to_add.append(path)

    archive = ZipFile(os.path.abspath(archive_path), 'w')

    for file in files_to_add:
        system_path = os.path.abspath(file)
        archive_path = plugin_name + '/' + file.replace('\\', '/').replace('./', '')
        archive.write(system_path, archive_path, ZIP_DEFLATED)
        print(archive_path)

    archive.close()


def main():
    from optparse import OptionParser
    parser = OptionParser()
    options, args = parser.parse_args()
    if len(args) != 1:
        parser.error("You must specify exactly one output file")
    output_file_path = args[0]
    if os.path.isdir(output_file_path):
        parser.error("You cannot use directory as output")

    create_package(output_file_path)

if __name__ == '__main__':
    main()
